# This file is a part of thelatinlibrary
#
# Copyright (c) 2018 The thelatinlibrary Authors (see AUTHORS)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from dataclasses import dataclass
from typing import List

import requests
from bs4 import BeautifulSoup

from thelatinlibrary import settings


@dataclass
class Tome:
    """
    The Tome class represents one of the
    multiple volumes used to write a book.
    """

    link: str
    """
    The link to the tome.
    """

    name: str
    """
    The name of the tome.
    """

    _text = ""
    """
    Used to store the result of the
    text getter.
    """

    @property
    def text(self) -> List[str]:
        """
        The text contained in the tome.
        """
        if settings.save and self._text:
            return self._text

        text = requests.get(self.link).text
        parser = BeautifulSoup(text, "html5lib")

        if '404' in parser.title.string:
            return []

        paragraphs = []
        texts = parser.find_all("p")

        for text in texts:
            paragraphs.append(text)

        if settings.save:
            self._text = paragraphs

        return paragraphs
